@@ -480,6 +480,21 @@ static void setup_path()
 	al_destroy( &l );
 }
 
+int env_set_pwd()
+{
+	wchar_t dir_path[4096];
+	wchar_t *res = wgetcwd( dir_path, 4096 );
+	if( !res )
+	{
+		return 0;
+	}
+	env_set( L"PWD", dir_path, ENV_EXPORT | ENV_GLOBAL );
+	return 1;
+}
+
+/**
+   Set up default values for various variables if not defined.
+ */
 static void env_set_defaults()
 {
 
@@ -502,6 +517,8 @@ static void env_set_defaults()
 		free( unam_narrow );
 	}	
 
+	env_set_pwd();
+	
 }
 
 void env_init()
@@ -1442,6 +1459,9 @@ static void export_func1( void *k, void *v, void *aux )
 	
 }
 
+/**
+   Get list of all exported variables
+ */
 static void get_exported( env_node_t *n, hash_table_t *h )
 {
 	if( !n )
