@@ -53,6 +53,11 @@
 */
 #define FD_ERROR   _( L"An error occurred while redirecting file descriptor %d" )
 
+/**
+   file descriptor redirection error message
+*/
+#define WRITE_ERROR   _( L"An error occurred while writing output" )
+
 /**
    file redirection error message
 */
@@ -92,6 +97,18 @@ static array_list_t *open_fds=0;
 
 static int set_child_group( job_t *j, process_t *p, int print_errors );
 
+static void exec_write_and_exit( int fd, char *buff, size_t count, int status )
+{
+	if( write_loop(fd, buff, count) == -1 ) 
+	{
+		debug( 0, WRITE_ERROR);
+		wperror( L"write" );
+		exit(status);
+	}
+	exit( status );
+}
+
+
 void exec_close( int fd )
 {
 	int i;
@@ -442,7 +459,41 @@ static int setup_child_process( job_t *j, process_t *p )
 	return res;
 	
 }
-								 
+
+/**
+   Returns the interpreter for the specified script. Returns 0 if file
+   is not a script with a shebang. This function leaks memory on every
+   call. Only use it in the execve error handler which calls exit
+   right afterwards, anyway.
+ */
+static wchar_t *get_interpreter( wchar_t *file )
+{
+	string_buffer_t sb;
+	FILE *fp = wfopen( file, "r" );
+	sb_init( &sb );
+	wchar_t *res = 0;
+	if( fp )
+	{
+		while( 1 )
+		{
+			wint_t ch = getwc( fp );
+			if( ch == WEOF )
+				break;
+			if( ch == L'\n' )
+				break;
+			sb_append_char( &sb, (wchar_t)ch );
+		}
+	}
+	
+	res = (wchar_t *)sb.buff;
+	
+	if( !wcsncmp( L"#! /", res, 4 ) )
+		return res+3;
+	if( !wcsncmp( L"#!/", res, 3 ) )
+		return res+2;
+	return 0;
+}
+
 								 
 /**
    This function is executed by the child process created by a call to
@@ -503,7 +554,7 @@ static void launch_process( process_t *p )
 			p->argv = res;
 			p->actual_cmd = L"/bin/sh";
 
-            res_real = wcsv2strv( (const wchar_t **) res);
+			res_real = wcsv2strv( (const wchar_t **) res);
 			
 			execve ( wcs2str(p->actual_cmd), 
 				 res_real,
@@ -549,22 +600,22 @@ static void launch_process( process_t *p )
 			if( arg_max > 0 )
 			{
 				
-				sb_format_size( &sz2, ARG_MAX );
+				sb_format_size( &sz2, arg_max );
 				
 				debug( 0,
-				       L"The total size of the argument and environment lists (%ls) exceeds the system limit of %ls.",
+				       L"The total size of the argument and environment lists (%ls) exceeds the operating system limit of %ls.",
 				       (wchar_t *)sz1.buff,
 				       (wchar_t *)sz2.buff);
 			}
 			else
 			{
 				debug( 0,
-				       L"The total size of the argument and environment lists (%ls) exceeds the system limit.",
+				       L"The total size of the argument and environment lists (%ls) exceeds the operating system limit.",
 				       (wchar_t *)sz1.buff);
 			}
 			
 			debug( 0, 
-			       L"Please try running the command again with fewer arguments.");
+			       L"Try running the command again with fewer arguments.");
 			sb_destroy( &sz1 );
 			sb_destroy( &sz2 );
 			
@@ -573,10 +624,42 @@ static void launch_process( process_t *p )
 			break;
 		}
 
+		case ENOEXEC:
+		{
+			wperror(L"exec");
+			
+			debug(0, L"The file '%ls' is marked as an executable but could not be run by the operating system.", p->actual_cmd);
+			exit(STATUS_EXEC_FAIL);
+		}
+
+		case ENOENT:
+		{
+			wchar_t *interpreter = get_interpreter( p->actual_cmd );
+			
+			if( interpreter && waccess( interpreter, X_OK ) )
+			{
+				debug(0, L"The file '%ls' specified the interpreter '%ls', which is not an executable command.", p->actual_cmd, interpreter );
+			}
+			else
+			{
+				debug(0, L"The file '%ls' or a script or ELF interpreter does not exist, or a shared library needed for file or interpreter cannot be found.", p->actual_cmd);
+			}
+			
+			exit(STATUS_EXEC_FAIL);
+		}
+
+		case ENOMEM:
+		{
+			debug(0, L"Out of memory");
+			exit(STATUS_EXEC_FAIL);
+		}
+
 		default:
 		{
-		  debug(0, L"The file '%ls' is marked as an executable but could not be run by the operating system.", p->actual_cmd);
-		  exit(STATUS_EXEC_FAIL);
+			wperror(L"exec");
+			
+			//		debug(0, L"The file '%ls' is marked as an executable but could not be run by the operating system.", p->actual_cmd);
+			exit(STATUS_EXEC_FAIL);
 		}
 	}
 	
@@ -830,6 +913,9 @@ static pid_t exec_fork()
 }
 
 
+/**
+   Perform output from builtins
+ */
 static void do_builtin_io( wchar_t *out, wchar_t *err )
 {
 
@@ -1357,15 +1443,17 @@ void exec( job_t *j )
 
 					if( pid == 0 )
 					{
+						
 						/*
 						  This is the child process. Write out the contents of the pipeline.
 						*/
 						p->pid = getpid();
 						setup_child_process( j, p );
-						write( io_buffer->fd, 
-							   io_buffer->param2.out_buffer->buff, 
-							   io_buffer->param2.out_buffer->used );
-						exit( status );
+
+						exec_write_and_exit(io_buffer->fd, 
+											io_buffer->param2.out_buffer->buff,
+											io_buffer->param2.out_buffer->used,
+											status);
 					}
 					else
 					{
@@ -1411,10 +1499,10 @@ void exec( job_t *j )
 					p->pid = getpid();
 					setup_child_process( j, p );
 					
-					write( 1,
-						   input_redirect->param2.out_buffer->buff, 
-						   input_redirect->param2.out_buffer->used );
-					exit( 0 );
+					exec_write_and_exit( 1,
+										 input_redirect->param2.out_buffer->buff, 
+										 input_redirect->param2.out_buffer->used,
+										 0);
 				}
 				else
 				{
@@ -1487,7 +1575,8 @@ void exec( job_t *j )
 					{
 						debug( 3, L"Set status of %ls to %d using short circut", j->command, p->status );
 						
-						proc_set_last_status( job_get_flag( j, JOB_NEGATE )?(!p->status):p->status );
+						int status = proc_format_status(p->status);
+						proc_set_last_status( job_get_flag( j, JOB_NEGATE )?(!status):status );
 					}
 					break;
 				}
